<?php

namespace Axepta\Payment;

/**
 * Class CartData
 *
 * @package Axepta\Payment
 */
class CartData
{

	/**
	 * @var array
	 */
	private $cart;

	/**
	 * @var array
	 */
	private $addressBilling;

	/**
	 * @var array
	 */
	private $addressDelivery;

	/**
	 * @var int
	 */
	private $grandTotal;

	/**
	 * @param string $name
	 * @param int    $price
	 * @param float  $quantity
	 * @param string $code
	 *
	 * @return void
	 */
	public function addItem($name, $price, $quantity, $code = '')
	{

		$item = [
			'name'     => $name,
			'price'    => $price,
			'quantity' => (float) $quantity,
		];

		if($code){
			$item['code'] = $code;
		}

		$this->cart[] = $item;
	}

	/**
	 * @param int $grandTotal
	 *
	 * @return void
	 */
	public function setGrandTotal($grandTotal)
	{

		$this->grandTotal = $grandTotal;
	}

	/**
	 * @param string $city
	 * @param string $street
	 * @param string $country
	 * @param string $postalCode
	 *
	 * @return void
	 */
	public function setAddressBilling($city, $street, $country, $postalCode)
	{
		$this->addressBilling = [
			'city'       => $city,
			'street'     => $street,
			'country'    => $country,
			'postalCode' => (string) $postalCode,
		];
	}

	/**
	 * @param string $city
	 * @param string $street
	 * @param string $country
	 * @param string $postalCode
	 *
	 * @return void
	 */
	public function setAddressDelivery($city, $street, $country, $postalCode)
	{
		$this->addressDelivery = [
			'city'       => $city,
			'street'     => $street,
			'country'    => $country,
			'postalCode' => (string) $postalCode,
		];
	}

	/**
	 * @return string
	 */
	public function prepareCartData()
	{

		return base64_encode(
			gzencode(
				json_encode(
					$this->prepareCartDataArray()
				),
				5
			)
		);
	}

	/**
	 * @return array
	 */
	private function prepareCartDataArray()
	{

		return [
			'customer'   => [
				'address' => [
					'billing'  => $this->addressBilling,
					'delivery' => $this->addressDelivery,
				],
			],
			'cart'       => $this->cart,
			'grandTotal' => $this->grandTotal,
		];
	}
}
