<?php

namespace Axepta\Paywall\Block;

include_once __DIR__ . "/../../Libs/payment-core/vendor/autoload.php";

use Axepta\Payment\CartData;
use Axepta\Payment\Paywall;
use Axepta\Payment\Util;
use Axepta\Paywall\Model\Order;
use Magento\Checkout\Helper\Cart;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\App\ResponseFactory;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\ScopeInterface;

/**
 * Class Checkout
 *
 * @package Axepta\Paywall\Block
 */
class Checkout extends Template
{

	/**
	 * @var UrlInterface
	 */
	public $urlBuilder;

	/**
	 * @var ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var Registry
	 */
	private $registry;

	/**
	 * @var Order
	 */
	private $order;

	/**
	 * @var StoreInterface
	 */
	private $store;

	/**
	 * @var Cart
	 */
	private $cartHelper;

	/**
	 * @var ResponseFactory
	 */
	private $responseFactory;

	/**
	 * @var ProductMetadataInterface $productMetadataInterface
	 */
	private $productMetadataInterface;

	/**
	 * Checkout constructor.
	 *
	 * @param Context                  $context
	 * @param Order                    $order
	 * @param Registry                 $registry
	 * @param StoreInterface           $store
	 * @param UrlInterface             $urlBuilder
	 * @param ScopeConfigInterface     $scopeConfig
	 * @param Cart                     $cartHelper
	 * @param ResponseFactory          $responseFactory
	 * @param ProductMetadataInterface $productMetadataInterface
	 *
	 * @internal param OrderFactory $orderFactory
	 */
	public function __construct(
		Context                  $context,
		Order                    $order,
		Registry                 $registry,
		StoreInterface           $store,
		UrlInterface             $urlBuilder,
		ScopeConfigInterface     $scopeConfig,
		Cart                     $cartHelper,
		ResponseFactory          $responseFactory,
		ProductMetadataInterface $productMetadataInterface

	) {
		parent::__construct($context);
		$this->registry = $registry;
		$this->order = $order;
		$this->store = $store;
		$this->urlBuilder = $urlBuilder;
		$this->scopeConfig = $scopeConfig;
		$this->cartHelper = $cartHelper;
		$this->responseFactory = $responseFactory;
		$this->productMetadataInterface = $productMetadataInterface;

		if($this->getOrderState($this->getOrderIdFromPost()) === \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT) {

			$this->redirectToHomePage();
			die();
		}
	}

	/**
	 * @param string $orderId
	 *
	 * @return mixed
	 */
	public function getOrderState($orderId = '')
	{
		return $this->getOrderObject($orderId)->getState();
	}

	/**
	 * Render form with order information prepared for Payment Page
	 *
	 * @param string $id
	 *
	 * @return bool|\Axepta\Paywall\Model\Sales\Order
	 */
	public function getOrderObject($id = '')
	{
		if(empty($id)) {
			$id = $this->getOrderId();
		}

		return $this->order->loadOrderById($id);
	}

	/**
	 * @return integer
	 */
	public function getOrderId()
	{
		return $this->registry->registry('orderId');
	}

	/**
	 * @return string
	 */
	public function getOrderIdFromPost()
	{
		$id = '';
		if(isset($_POST['orderId'])) {
			$id = $_POST['orderId'];
		}

		return $id;
	}

	/**
	 * That function redirect to homepage
	 *
	 * @return mixed
	 */
	public function redirectToHomePage()
	{
		return $this->responseFactory->create()->setRedirect($this->urlBuilder->getUrl())->sendResponse();
	}

	/**
	 * @param string $orderId
	 *
	 * @return string
	 */
	public function renderForm($orderId = '')
	{

		if(empty($orderId)) {
			$orderId = $this->getOrderId();
		}

		$order = $this->getOrderObject($orderId);

		$order->setState(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT);
		$order->save();

		/**
		 * Get Magento version.
		 */
		$version = $this->productMetadataInterface->getVersion();

		$payment = new Paywall(
			$this->getEnvironment(),
			$this->getConfigValue('payment/axepta_paywall/service_key'),
			$this->getConfigValue('payment/axepta_paywall/service_id'),
			$this->getConfigValue('payment/axepta_paywall/merchant_id')
		);

		$orderData = Paywall::prepareData(
			Util::convertAmountToFractional($order->getGrandTotal()),
			$order->getOrderCurrency()->getCode(),
			$order->getIncrementId(),
			$order->getBillingAddress()->getFirstname(),
			$order->getBillingAddress()->getLastname(),
			$order->getCustomerEmail(),
			$order->getBillingAddress()->getTelephone(),
			$this->urlBuilder->getUrl('checkout/onepage/success/'),
			$this->urlBuilder->getUrl('checkout/onepage/failure/'),
			$this->urlBuilder->getUrl('checkout/onepage/failure/'),
			$this->getConfigValue('payment/axepta_paywall/credit_type'),
			$this->getCartData($order),
			$this->getConfigValue('payment/axepta_paywall/version') . ';magento2_' . $version
		);

		return $payment->buildOrderForm($orderData, __('Continue'));
	}

	/**
	 * @return string
	 */
	public function getEnvironment()
	{
		return $this->getConfigValue('payment/axepta_paywall/sandbox') == 1
			? Util::ENVIRONMENT_SANDBOX
			: Util::ENVIRONMENT_PRODUCTION;
	}

	/**
	 * @param string $value
	 *
	 * @return string
	 */
	private function getConfigValue($value)
	{
		return $this->scopeConfig->getValue($value, ScopeInterface::SCOPE_STORE);
	}

	/**
	 * @param Order $order
	 *
	 * @return string
	 */
	private function getCartData($order)
	{

		$cartData = new CartData();

		foreach($order->getAllItems() as $item) {

			$product = $item->getProduct();

			$materialCode = '';

			if(isset($product['mat_code']) && $product['mat_code']) {
				$materialCode = $product['mat_code'];
			}

			$cartData->addItem(
				$item->getName(),
				Util::convertAmountToFractional($item->getPriceInclTax()),
				(float) $item->getQtyOrdered(),
				$materialCode
			);
		}

		$cartData->setGrandTotal(Util::convertAmountToFractional($order->getGrandTotal()));

		$addressBilling = $order->getBillingAddress();
		$cartData->setAddressBilling(
			$addressBilling->getCity(),
			$addressBilling->getStreet()[0] . (empty($addressBilling->getStreet()[1])
				? ''
				: ', ' . $addressBilling->getStreet()[1]),
			$addressBilling->getCountryId(),
			$addressBilling->getPostcode()
		);

		$addressDelivery = $order->getShippingAddress();
		$cartData->setAddressDelivery(
			$addressDelivery->getCity(),
			$addressDelivery->getStreet()[0] . (empty($addressDelivery->getStreet()[1])
				? ''
				: ', ' . $addressDelivery->getStreet()[1]),
			$addressDelivery->getCountryId(),
			$addressDelivery->getPostcode()
		);

		return $cartData->prepareCartData();
	}
}
