<?php

namespace Axepta\Paywall\Controller\Payment;

include_once __DIR__ . "/../../../Libs/payment-core/vendor/autoload.php";

use Axepta\Payment\Util;
use Exception;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Email\Sender\InvoiceSender;
use Magento\Sales\Model\Order\Invoice;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;
use Magento\Sales\Model\OrderRepository;
use Magento\Sales\Model\Service\InvoiceService;
use Magento\Store\Model\ScopeInterface;
use Magento\Sales\Model\Service\CreditmemoService;
use Magento\Sales\Model\Order\CreditmemoFactory;
use Magento\Sales\Api\OrderStatusHistoryRepositoryInterface;

/**
 * Class Notification
 *
 * @package Axepta\Paywall\Controller\Payment
 */
class Notification extends Action implements CsrfAwareActionInterface
{

	/**
	 * @var Order
	 */
	protected $order;

	/**
	 * @var ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var BuilderInterface
	 */
	protected $builderInterface;

	/**
	 * @var OrderRepository
	 */
	protected $orderRepository;

	/**
	 * @var InvoiceService
	 */
	protected $invoiceService;

	/**
	 * @var \Magento\Framework\DB\Transaction
	 */
	protected $dbTransaction;

	/**
	 * @var Magento\Sales\Model\Order\Email\Sender\InvoiceSender
	 */
	protected $invoiceSender;

	/**
	 * @var Magento\Sales\Model\Order\CreditmemoFactory
	 */
	protected $creditmemoFactory;

	/**
	 * @var Magento\Sales\Model\Service\CreditmemoService
	 */
	protected $creditmemoService;

	/**
	 * @var OrderStatusHistoryRepositoryInterface
	 */
	protected $orderStatusRepository;

	/**
	 * @inheritDoc
	 */
	public function createCsrfValidationException($request)
	{
		return null;
	}

	/**
	 * @inheritDoc
	 */
	public function validateForCsrf($request)
	{
		return true;
	}

	/**
	 * ReturnUrl constructor.
	 *
	 * @param Context                               $context
	 * @param Order                                 $order
	 * @param ScopeConfigInterface                  $scopeConfig
	 * @param BuilderInterface                      $builderInterface
	 * @param OrderRepositoryInterface              $orderRepository
	 * @param InvoiceService                        $invoiceService
	 * @param \Magento\Framework\DB\Transaction     $dbTransaction
	 * @param InvoiceSender                         $invoiceSender
	 * @param CreditmemoFactory                     $creditmemoFactory
	 * @param CreditmemoService                     $creditmemoService
	 * @param OrderStatusHistoryRepositoryInterface $orderStatusRepository
	 */
	public function __construct(
		Context                               $context,
		Order                                 $order,
		ScopeConfigInterface                  $scopeConfig,
		BuilderInterface                      $builderInterface,
		OrderRepositoryInterface              $orderRepository,
		InvoiceService                        $invoiceService,
		\Magento\Framework\DB\Transaction     $dbTransaction,
		InvoiceSender                         $invoiceSender,
		CreditmemoFactory                     $creditmemoFactory,
		CreditmemoService                     $creditmemoService,
		OrderStatusHistoryRepositoryInterface $orderStatusRepository
	) {
		parent::__construct($context);
		$this->order = $order;
		$this->scopeConfig = $scopeConfig;
		$this->builderInterface = $builderInterface;
		$this->orderRepository = $orderRepository;
		$this->invoiceService = $invoiceService;
		$this->dbTransaction = $dbTransaction;
		$this->invoiceSender = $invoiceSender;
		$this->creditmemoFactory = $creditmemoFactory;
		$this->creditmemoService = $creditmemoService;
		$this->orderStatusRepository = $orderStatusRepository;
	}

	/**
	 * @return ResponseInterface|ResultInterface|void
	 * @throws Exception
	 */
	public function execute()
	{

		$notification = new \Axepta\Payment\Notification(
			$this->getConfigValue('payment/axepta_paywall/service_id'),
			$this->getConfigValue('payment/axepta_paywall/service_key')
		);

		// it can be order data or notification code - depends on verification notification
		$resultCheckRequestNotification = $notification->checkRequest();

		if(is_int($resultCheckRequestNotification)) {

			echo \Axepta\Payment\Notification::formatResponse(
				\Axepta\Payment\Notification::NS_ERROR,
				$resultCheckRequestNotification);
			exit();
		}

		$order = $this->order->loadByIncrementId($resultCheckRequestNotification['payment']['orderId']);

		if($order->getEntityId() === null) {

			echo \Axepta\Payment\Notification::formatResponse(
				\Axepta\Payment\Notification::NS_ERROR,
				\Axepta\Payment\Notification::NC_ORDER_NOT_FOUND
			);
			exit();
		}

		if($resultCheckRequestNotification['payment']['notifyTransactionData']['type'] === \Axepta\Payment\Api::TRANSACTION_TYPE_REFUND) {

			$currentOrderStatus = $order->getState();

			if($currentOrderStatus === Order::STATE_CLOSED) {

				echo \Axepta\Payment\Notification::formatResponse(
					\Axepta\Payment\Notification::NS_ERROR,
					\Axepta\Payment\Notification::NC_INVALID_ORDER_STATUS
				);
				exit();
			}

			$transactionStatus = '';
			$amountToRefund = 0;

			foreach($resultCheckRequestNotification['payment']['transactions'] as $transaction) {

				if($transaction['id'] === $resultCheckRequestNotification['payment']['notifyTransactionData']['id']) {
					$amountToRefund = Util::convertAmountToMain($transaction['amount']);
					$transactionStatus = $transaction['status'];
					break;
				}
			}

			if($transactionStatus !== \Axepta\Payment\Notification::PS_SETTLED) {
				echo \Axepta\Payment\Notification::formatResponse(\Axepta\Payment\Notification::NS_OK);
				exit();
			}

			try {
				$creditmemo = $this->creditmemoFactory->createByOrder($order);
				$creditmemo->setGrandTotal($amountToRefund);
				$this->creditmemoService->refund($creditmemo);
				$this->creditmemoService->notify($creditmemo->getId());

			} catch(\Exception $e) {

				echo \Axepta\Payment\Notification::formatResponse(
					\Axepta\Payment\Notification::NS_ERROR
				);
				exit();
			}

			echo \Axepta\Payment\Notification::formatResponse(\Axepta\Payment\Notification::NS_OK);
			exit();
		}

		if(!\Axepta\Payment\Notification::checkRequestAmount(
			$resultCheckRequestNotification,
			Util::convertAmountToFractional($order->getGrandTotal()),
			$order->getOrderCurrencyCode()
		)) {
			echo \Axepta\Payment\Notification::formatResponse(
				\Axepta\Payment\Notification::NS_ERROR,
				\Axepta\Payment\Notification::NC_AMOUNT_NOT_MATCH);
			exit();
		}

		$currentOrderStatus = $order->getState();

		if($currentOrderStatus === Order::STATE_PROCESSING) {

			echo \Axepta\Payment\Notification::formatResponse(
				\Axepta\Payment\Notification::NS_OK,
				\Axepta\Payment\Notification::NC_ORDER_STATUS_NOT_CHANGED
			);
			exit();
		}

		switch($resultCheckRequestNotification['payment']['status']) {
			case \Axepta\Payment\Notification::PS_SETTLED:
				$order->setTotalPaid(Util::convertAmountToMain($resultCheckRequestNotification['payment']['amount']));
				$this->createTransaction($order, $resultCheckRequestNotification);
				$this->createInvoice($order, $resultCheckRequestNotification['payment']['id']);
				$order->save();
				break;
			case \Axepta\Payment\Notification::PS_REJECTED:
				$orderState = Order::STATE_PAYMENT_REVIEW;
				$order->setState($orderState)->setStatus($orderState);
				$order->save();
				$order->addStatusToHistory($orderState, __('The payment has been rejected.'));
				break;
			default:
				break;
		}

		echo \Axepta\Payment\Notification::formatResponse(\Axepta\Payment\Notification::NS_OK);
		exit();
	}

	/**
	 * @param string $value
	 *
	 * @return string
	 */
	private function getConfigValue($value)
	{
		return $this->scopeConfig->getValue($value, ScopeInterface::SCOPE_STORE);
	}

	/**
	 * @param Order $order
	 * @param array $payloadDecoded
	 *
	 * @return bool
	 */
	public function createTransaction($order, $payloadDecoded)
	{
		//get payment object from order object
		//get the object of builder class
		if(
			!($payment = $order->getPayment())
			|| !($trans = $this->builderInterface)
		) {
			return false;
		}

		$transaction = $trans->setPayment($payment)
			->setOrder($order)
			->setTransactionId($payloadDecoded['payment']['id'])
			->setFailSafe(true)
			//build method creates the transaction and returns the object
			->build(Transaction::TYPE_CAPTURE);
		if(!$transaction) {
			return false;
		}

		$payment->addTransactionCommentsToOrder(
			$transaction,
			__('Paid amount: %1.', Util::convertAmountToMain($payloadDecoded['payment']['amount']))
		);

		$payment->setTransactionId($payloadDecoded['payment']['id']);

		$payment->setParentTransactionId(null);

		try {
			$payment->save();
			$transaction->save();

			return true;
		} catch(Exception $e) {
			return false;
		}
	}

	/**
	 * @param Order  $order
	 * @param string $transactionId
	 *
	 * @return bool
	 */
	public function createInvoice($order, $transactionId)
	{

		if(!$order->canInvoice()) {

			return false;
		}

		$invoice = $order->prepareInvoice();
		$invoice->setRequestedCaptureCase(Invoice::CAPTURE_ONLINE);
		$invoice->setTransactionId($transactionId);
		$invoice->register();
		$invoice->pay();
		$invoice->save();

		if($invoice && !$invoice->getEmailSent()) {

			$this->invoiceSender->send($invoice);

			$order->addRelatedObject($invoice);
			$order->addStatusHistoryComment(__('You notified customer about invoice #%1.', $invoice->getIncrementId()))
				->setIsCustomerNotified(true)
				->save();

			return true;
		}

		return false;
	}
}
